==> react-router-1.11.0.tgz: package/lib/internal/createTransitionManager.js <==
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { formatPattern, matchPattern } from '../PatternUtils';
import internalIsActive from './isActive';
import { Tree } from './Tree';
import warning from './warning';
var INITIAL_MATCH_OBJECT = {
  isFinished: false,
  nodes: [],
  paramNames: [],
  paramValues: []
};

var checkIntegrity = function checkIntegrity(nodes) {
  return !nodes.some(function (node, index) {
    return node.level !== index + 1;
  });
};

var createParams = function createParams(_ref) {
  var paramValues = _ref.paramValues,
      paramNames = _ref.paramNames;
  return paramNames.reduce(function (acc, name, index) {
    var _extends2;

    return _extends({}, acc, (_extends2 = {}, _extends2[name] = paramValues[index], _extends2));
  }, {});
};
/**
 * Find if there's a Redirect route
 * @param {Array} nodes Array of nodes to check
 */


var findRedirect = function findRedirect(nodes) {
  // get last node
  var tail = nodes[nodes.length - 1]; // if last node is a redirect, we return it

  return tail && tail.redirect ? tail : null;
};
/**
 * Get a component passing the route info and returning a promise with the result
 * @param {(location: object, callback: Function) => Promise<object>} getComponent Function to retrieve a component async
 * @param {import('../types').RouteInfo} routeInfo
 * @returns {Promise<import('react').ComponentType>}
 */


var makePromise = function makePromise(getComponent, routeInfo) {
  return new Promise(function (resolve, reject) {
    getComponent(routeInfo, function (err, component) {
      return err ? reject(err) : resolve(component);
    });
  });
};

var createComponents = function createComponents(_ref2) {
  var nodes = _ref2.nodes,
      routeInfo = _ref2.routeInfo;
  var indexRoute = nodes.findIndex(function (node) {
    return node.fromIndex;
  });

  if (indexRoute !== -1 && indexRoute === nodes.length - 1) {
    nodes = [].concat(nodes, [nodes[indexRoute].indexNode]);
  } // create an array of promises with all components


  var componentsPromises = nodes.filter(function (node) {
    return node.component || node.getComponent;
  }).map(function (_ref3) {
    var component = _ref3.component,
        getComponent = _ref3.getComponent;
    return component ? Promise.resolve(component) : makePromise(getComponent, routeInfo);
  }); // return a promise that will resolve when all components are available

  return Promise.all(componentsPromises);
};
/**
 * Create the reducer to transverse the three
 * @param {import('../types').Location} location
 * @returns {(acc: object, node: object) => object}
 */


var createReducerRoutesTree = function createReducerRoutesTree(location) {
  return function (acc, node) {
    var _acc = acc,
        remainingPathname = _acc.remainingPathname,
        paramNames = _acc.paramNames,
        paramValues = _acc.paramValues;
    if (acc.isFinished) return acc;
    if (node.index) return acc;
    var pattern = node.path,
        regexp = node.regexp;

    if (!regexp && !pattern) {
      pattern = '';
    }

    acc = _extends({}, acc, {
      nodes: acc.nodes.filter(function (n) {
        return n.level < node.level;
      })
    });

    if (regexp) {
      var match = remainingPathname.match(regexp);

      if (match) {
        acc = _extends({}, acc, {
          remainingPathname: match.input.replace(match[0], ''),
          nodes: [].concat(acc.nodes, [node])
        });

        if (match.groups) {
          acc.paramNames = [].concat(paramNames, Object.keys(match.groups));
          acc.paramValues = [].concat(paramNames, Object.values(match.groups));
        }

        if (acc.remainingPathname === '') {
          acc.isFinished = checkIntegrity(acc.nodes);
        }
      }

      return acc;
    }

    if (pattern.charAt(0) === '/') {
      remainingPathname = location.pathname;
      paramNames = [];
      paramValues = [];
    }

    if (pattern === '') {
      return _extends({}, acc, {
        nodes: [].concat(acc.nodes, [node])
      });
    }

    var matched = matchPattern(pattern, remainingPathname);

    if (matched) {
      acc = {
        nodes: [].concat(acc.nodes, [node]),
        paramNames: [].concat(paramNames, matched.paramNames),
        paramValues: [].concat(paramValues, matched.paramValues),
        remainingPathname: matched.remainingPathname
      };

      if (matched.remainingPathname === '') {
        acc.isFinished = checkIntegrity(acc.nodes);
      }
    }

    return acc;
  };
};
/**
 * Reduce the tree and check if the current location is matching the routes keeping in mind the remainingPathname
 * @param {Object} tree
 * @param {import('../types').Location} location
 * @param {String=} remainingPathname
 * @returns {Promise<Object>}
 */


var matchRoutes = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(tree, location, remainingPathname) {
    var initialObject, match, nodesFromMatch, paramValues, paramNames, params, redirectNode, nodes, routeInfo, components;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (remainingPathname === undefined) {
              if (location.pathname.charAt(0) !== '/') {
                location = _extends({}, location, {
                  pathname: "/" + location.pathname
                });
              }

              remainingPathname = location.pathname;
            }

            initialObject = _extends({}, INITIAL_MATCH_OBJECT, {
              remainingPathname: remainingPathname
            });
            match = Tree.reduce(createReducerRoutesTree(location), initialObject, tree);
            nodesFromMatch = match.nodes, paramValues = match.paramValues, paramNames = match.paramNames;
            params = createParams({
              paramNames: paramNames,
              paramValues: paramValues
            }); // check if we have hit a redirect

            redirectNode = findRedirect(nodesFromMatch);

            if (!redirectNode) {
              _context.next = 8;
              break;
            }

            return _context.abrupt("return", {
              redirectLocation: formatPattern(redirectNode.to, params)
            });

          case 8:
            // if it's not a redirect and still there's remainingPathname then is not a match
            nodes = match.remainingPathname ? INITIAL_MATCH_OBJECT.nodes : nodesFromMatch;
            routeInfo = {
              location: location,
              params: params,
              routes: nodes
            };
            _context.next = 12;
            return createComponents({
              nodes: nodes,
              routeInfo: routeInfo
            });

          case 12:
            components = _context.sent;
            return _context.abrupt("return", {
              routeInfo: routeInfo,
              components: components
            });

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function matchRoutes(_x, _x2, _x3) {
    return _ref4.apply(this, arguments);
  };
}();

export var createTransitionManager = function createTransitionManager(_ref5) {
  var history = _ref5.history,
      jsonRoutes = _ref5.jsonRoutes;

  /** @type {import('../types').RouterState} */
  var state = {};

  var matchRouteAndUpdateState = /*#__PURE__*/function () {
    var _ref7 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(_ref6) {
      var location, jsonRoutes, _yield$matchRoutes, redirectLocation, routeInfo, components;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              location = _ref6.location, jsonRoutes = _ref6.jsonRoutes;
              _context2.next = 3;
              return matchRoutes(jsonRoutes, location);

            case 3:
              _yield$matchRoutes = _context2.sent;
              redirectLocation = _yield$matchRoutes.redirectLocation;
              routeInfo = _yield$matchRoutes.routeInfo;
              components = _yield$matchRoutes.components;
              state = _extends({}, state, routeInfo, {
                components: components,
                location: location
              });
              return _context2.abrupt("return", {
                redirectLocation: redirectLocation,
                routeInfo: routeInfo,
                components: components
              });

            case 9:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function matchRouteAndUpdateState(_x4) {
      return _ref7.apply(this, arguments);
    };
  }();
  /**
   * Match the current location with the json routes
   * @param {import('../types').Location} location Current location object
   */


  var match = function match(location) {
    location = location ? history.createLocation(location) : history.getCurrentLocation();
    return matchRouteAndUpdateState({
      jsonRoutes: jsonRoutes,
      location: location
    });
  };
  /**
   * This is the API for stateful environments. As the location
   * changes, we update state and call the listener. We can also
   * gracefully handle errors and redirects.
   */


  var listen = function listen(listener) {
    var historyListener = /*#__PURE__*/function () {
      var _ref8 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(location) {
        var _yield$matchRouteAndU, redirectLocation, components, url;

        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(state.location === location)) {
                  _context3.next = 4;
                  break;
                }

                listener(null, {
                  params: state.params,
                  components: state.components
                });
                _context3.next = 19;
                break;

              case 4:
                _context3.prev = 4;
                _context3.next = 7;
                return matchRouteAndUpdateState({
                  jsonRoutes: jsonRoutes,
                  location: location
                });

              case 7:
                _yield$matchRouteAndU = _context3.sent;
                redirectLocation = _yield$matchRouteAndU.redirectLocation;
                components = _yield$matchRouteAndU.components;

                if (!redirectLocation) {
                  _context3.next = 12;
                  break;
                }

                return _context3.abrupt("return", history.replace(redirectLocation));

              case 12:
                if (!components) {
                  url = location.pathname + location.search + location.hash;
                  warning(false, "Location \"" + url + "\" did not match any routes");
                } // call listener with null errors and the nextState


                return _context3.abrupt("return", listener(null, state));

              case 16:
                _context3.prev = 16;
                _context3.t0 = _context3["catch"](4);
                listener(_context3.t0);

              case 19:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, null, [[4, 16]]);
      }));

      return function historyListener(_x5) {
        return _ref8.apply(this, arguments);
      };
    }(); // TODO: Only use a single history listener. Otherwise we'll end up with
    // multiple concurrent calls to match.
    // Set up the history listener first in case the initial match redirects.


    var unsubscribe = history.listen(historyListener);

    if (state.location) {
      listener(null, state); // Picking up on a matchContext
    } else {
      historyListener(history.getCurrentLocation());
    }

    return unsubscribe;
  };
  /**
   * Check if current location is the active one using the history
   * @param {import('../types').Location} location
   * @param {Boolean} indexOnly Should we check only on index path?
   * @returns {Boolean}
   */


  var isActive = function isActive(location, indexOnly) {
    location = history.createLocation(location);
    return internalIsActive(location, indexOnly, state.location, state.routes, state.params);
  };

  return {
    match: match,
    listen: listen,
    isActive: isActive
  };
};

==> react-router-1.14.0.tgz: package/lib/internal/createTransitionManager.js <==
import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { formatPattern, matchPattern } from '../PatternUtils.js';
import internalIsActive from './isActive.js';
import { Tree } from './Tree.js';
import warning from './warning.js';
var INITIAL_MATCH_OBJECT = {
  isFinished: false,
  nodes: [],
  paramNames: [],
  paramValues: []
};

var checkIntegrity = function checkIntegrity(nodes) {
  return !nodes.some(function (node, index) {
    return node.level !== index + 1;
  });
};

var createParams = function createParams(_ref) {
  var paramValues = _ref.paramValues,
      paramNames = _ref.paramNames;
  return paramNames.reduce(function (acc, name, index) {
    var _extends2;

    return _extends({}, acc, (_extends2 = {}, _extends2[name] = paramValues[index], _extends2));
  }, {});
};
/**
 * Find if there's a Redirect route
 * @param {Array} nodes Array of nodes to check
 */


var findRedirect = function findRedirect(nodes) {
  // get last node
  var tail = nodes[nodes.length - 1]; // if last node is a redirect, we return it

  return tail && tail.redirect ? tail : null;
};
/**
 * Get a component passing the route info and returning a promise with the result
 * @param {(location: object, callback: Function) => Promise<object>} getComponent Function to retrieve a component async
 * @param {import('../types').RouteInfo} routeInfo
 * @returns {Promise<import('react').ComponentType>}
 */


var makePromise = function makePromise(getComponent, routeInfo) {
  return new Promise(function (resolve, reject) {
    getComponent(routeInfo, function (err, component) {
      return err ? reject(err) : resolve(component);
    });
  });
};

var createComponents = function createComponents(_ref2) {
  var nodes = _ref2.nodes,
      routeInfo = _ref2.routeInfo;
  var indexRoute = nodes.findIndex(function (node) {
    return node.fromIndex;
  });

  if (indexRoute !== -1 && indexRoute === nodes.length - 1) {
    nodes = [].concat(nodes, [nodes[indexRoute].indexNode]);
  } // create an array of promises with all components


  var componentsPromises = nodes.filter(function (node) {
    return node.component || node.getComponent;
  }).map(function (_ref3) {
    var component = _ref3.component,
        getComponent = _ref3.getComponent;
    return component ? Promise.resolve(component) : makePromise(getComponent, routeInfo);
  }); // return a promise that will resolve when all components are available

  return Promise.all(componentsPromises);
};
/**
 * Create the reducer to transverse the three
 * @param {import('../types').Location} location
 * @returns {(acc: object, node: object) => object}
 */


var createReducerRoutesTree = function createReducerRoutesTree(location) {
  return function (acc, node) {
    var _acc = acc,
        remainingPathname = _acc.remainingPathname,
        paramNames = _acc.paramNames,
        paramValues = _acc.paramValues;
    if (acc.isFinished) return acc;
    if (node.index) return acc;
    var pattern = node.path,
        regexp = node.regexp;

    if (!regexp && !pattern) {
      pattern = '';
    }

    acc = _extends({}, acc, {
      nodes: acc.nodes.filter(function (n) {
        return n.level < node.level;
      })
    });

    if (regexp) {
      var match = remainingPathname.match(regexp);

      if (match) {
        acc = _extends({}, acc, {
          remainingPathname: match.input.replace(match[0], ''),
          nodes: [].concat(acc.nodes, [node])
        });

        if (match.groups) {
          acc.paramNames = [].concat(paramNames, Object.keys(match.groups));
          acc.paramValues = [].concat(paramNames, Object.values(match.groups));
        }

        if (acc.remainingPathname === '') {
          acc.isFinished = checkIntegrity(acc.nodes);
        }
      }

      return acc;
    }

    if (pattern.charAt(0) === '/') {
      remainingPathname = location.pathname;
      paramNames = [];
      paramValues = [];
    }

    if (pattern === '') {
      return _extends({}, acc, {
        nodes: [].concat(acc.nodes, [node])
      });
    }

    var matched = matchPattern(pattern, remainingPathname);

    if (matched) {
      acc = {
        nodes: [].concat(acc.nodes, [node]),
        paramNames: [].concat(paramNames, matched.paramNames),
        paramValues: [].concat(paramValues, matched.paramValues),
        remainingPathname: matched.remainingPathname
      };

      if (matched.remainingPathname === '') {
        acc.isFinished = checkIntegrity(acc.nodes);
      }
    }

    return acc;
  };
};
/**
 * Reduce the tree and check if the current location is matching the routes keeping in mind the remainingPathname
 * @param {Object} tree
 * @param {import('../types').Location} location
 * @param {String=} remainingPathname
 * @returns {Promise<Object>}
 */


var matchRoutes = /*#__PURE__*/function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(tree, location, remainingPathname) {
    var initialObject, match, nodesFromMatch, paramValues, paramNames, params, redirectNode, nodes, routeInfo, components;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (remainingPathname === undefined) {
              if (location.pathname.charAt(0) !== '/') {
                location = _extends({}, location, {
                  pathname: "/" + location.pathname
                });
              }

              remainingPathname = location.pathname;
            }

            initialObject = _extends({}, INITIAL_MATCH_OBJECT, {
              remainingPathname: remainingPathname
            });
            match = Tree.reduce(createReducerRoutesTree(location), initialObject, tree);
            nodesFromMatch = match.nodes, paramValues = match.paramValues, paramNames = match.paramNames;
            params = createParams({
              paramNames: paramNames,
              paramValues: paramValues
            }); // check if we have hit a redirect

            redirectNode = findRedirect(nodesFromMatch);

            if (!redirectNode) {
              _context.next = 8;
              break;
            }

            return _context.abrupt("return", {
              redirectLocation: formatPattern(redirectNode.to, params)
            });

          case 8:
            // if it's not a redirect and still there's remainingPathname then is not a match
            nodes = match.remainingPathname ? INITIAL_MATCH_OBJECT.nodes : nodesFromMatch;
            routeInfo = {
              location: location,
              params: params,
              routes: nodes
            };
            _context.next = 12;
            return createComponents({
              nodes: nodes,
              routeInfo: routeInfo
            });

          case 12:
            components = _context.sent;
            return _context.abrupt("return", {
              routeInfo: routeInfo,
              components: components
            });

          case 14:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function matchRoutes(_x, _x2, _x3) {
    return _ref4.apply(this, arguments);
  };
}();

export var createTransitionManager = function createTransitionManager(_ref5) {
  var history = _ref5.history,
      jsonRoutes = _ref5.jsonRoutes;

  /** @type {import('../types').RouterState} */
  var state = {};

  var matchRouteAndUpdateState = /*#__PURE__*/function () {
    var _ref7 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(_ref6) {
      var location, jsonRoutes, _yield$matchRoutes, redirectLocation, routeInfo, components;

      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              location = _ref6.location, jsonRoutes = _ref6.jsonRoutes;
              _context2.next = 3;
              return matchRoutes(jsonRoutes, location);

            case 3:
              _yield$matchRoutes = _context2.sent;
              redirectLocation = _yield$matchRoutes.redirectLocation;
              routeInfo = _yield$matchRoutes.routeInfo;
              components = _yield$matchRoutes.components;
              state = _extends({}, state, routeInfo, {
                components: components,
                location: location
              });
              return _context2.abrupt("return", {
                redirectLocation: redirectLocation,
                routeInfo: routeInfo,
                components: components
              });

            case 9:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2);
    }));

    return function matchRouteAndUpdateState(_x4) {
      return _ref7.apply(this, arguments);
    };
  }();
  /**
   * Match the current location with the json routes
   * @param {import('../types').Location} location Current location object
   */


  var match = function match(location) {
    location = location ? history.createLocation(location) : history.getCurrentLocation();
    return matchRouteAndUpdateState({
      jsonRoutes: jsonRoutes,
      location: location
    });
  };
  /**
   * This is the API for stateful environments. As the location
   * changes, we update state and call the listener. We can also
   * gracefully handle errors and redirects.
   */


  var listen = function listen(listener) {
    var historyListener = /*#__PURE__*/function () {
      var _ref8 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(location) {
        var _yield$matchRouteAndU, redirectLocation, components, url;

        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                if (!(state.location === location)) {
                  _context3.next = 4;
                  break;
                }

                listener(null, {
                  params: state.params,
                  components: state.components
                });
                _context3.next = 19;
                break;

              case 4:
                _context3.prev = 4;
                _context3.next = 7;
                return matchRouteAndUpdateState({
                  jsonRoutes: jsonRoutes,
                  location: location
                });

              case 7:
                _yield$matchRouteAndU = _context3.sent;
                redirectLocation = _yield$matchRouteAndU.redirectLocation;
                components = _yield$matchRouteAndU.components;

                if (!redirectLocation) {
                  _context3.next = 12;
                  break;
                }

                return _context3.abrupt("return", history.replace(redirectLocation));

              case 12:
                if (!components) {
                  url = location.pathname + location.search + location.hash;
                  warning(false, "Location \"" + url + "\" did not match any routes");
                } // call listener with null errors and the nextState


                return _context3.abrupt("return", listener(null, state));

              case 16:
                _context3.prev = 16;
                _context3.t0 = _context3["catch"](4);
                listener(_context3.t0);

              case 19:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3, null, [[4, 16]]);
      }));

      return function historyListener(_x5) {
        return _ref8.apply(this, arguments);
      };
    }(); // TODO: Only use a single history listener. Otherwise we'll end up with
    // multiple concurrent calls to match.
    // Set up the history listener first in case the initial match redirects.


    var unsubscribe = history.listen(historyListener);

    if (state.location) {
      listener(null, state); // Picking up on a matchContext
    } else {
      historyListener(history.getCurrentLocation());
    }

    return unsubscribe;
  };
  /**
   * Check if current location is the active one using the history
   * @param {import('../types').Location} location
   * @param {Boolean} indexOnly Should we check only on index path?
   * @returns {Boolean}
   */


  var isActive = function isActive(location, indexOnly) {
    location = history.createLocation(location);
    return internalIsActive(location, indexOnly, state.location, state.routes, state.params);
  };

  return {
    match: match,
    listen: listen,
    isActive: isActive
  };
};
